/* $Id$ */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/utsname.h>
#include <pwd.h>
#include <string.h>

#include "arch.h"


/* Function to detect the current architecture */
const char *detect_arch(void)
{
    const char *arch;

    /* See if there is an environment override */
    arch = getenv("SETUP_ARCH");
    if ( arch == NULL ) {
#ifdef __i386
        arch = "x86";
#elif defined(powerpc)
        arch = "ppc";
#elif defined(__alpha__)
        arch = "alpha";
#elif defined(__sparc__)
        arch = "sparc64";
#elif defined(__arm__)
        arch = "arm";
#else
        arch = "unknown";
#endif
    }
    return arch;
}

/* Function to detect the operating system */
const char *detect_os(void)
{
    static struct utsname buf;

    uname(&buf);
    return buf.sysname;
}

/* Function to detect the current version of libc */
const char *detect_libc(void)
{
#ifdef __FreeBSD__
	return "glibc-2.2";
#else
    static const char *libclist[] = {
        "/lib32",
        "/emul/ia32-linux/lib",
        "/lib",
        NULL
    };
    int i;
    const char *libc;
    const char *libcdir;

    /* See if there is an environment override */
    libc = getenv("SETUP_LIBC");
    if ( libc != NULL ) {
        return(libc);
    }

    /* Look for the first of the specified dirs */
    for ( i=0; libclist[i]; ++i ) {
        if ( access(libclist[i], F_OK) == 0 ) {
            break;
        }
    }
    libcdir = libclist[i];

    if ( libcdir ) {
        /* FIXME: We should not have to masquerade a glibc 2.3+ system
         *        as a glibc 2.2 one.
         */
        char buffer[1024];
        snprintf( buffer, sizeof(buffer), 
                  "fgrep GLIBC_2.2 %s/libc.so.6* >/dev/null 2>&1",
                  libcdir );
        if ( system(buffer) == 0 )
            return "glibc-2.2";
        snprintf( buffer, sizeof(buffer), 
                  "fgrep GLIBC_2.1 %s/libc.so.6* 2>&1 >/dev/null",
                  libcdir );
        if ( system(buffer) == 0 )
            return "glibc-2.1";
        return "glibc-2.0";
    }
    /* Default to version 5 */
    return "libc5";
#endif
}

/* Function that returns the current user's home directory */
const char *detect_home(void)
{
    static const char *home = NULL;

    /* Try the HOME environment variable.
     */
    if ( ! home ) {
        home = getenv("HOME");
    }

    /* Look up the user's home directory in the password file,
       based on effective user id.
     */
    if ( ! home ) {
        struct passwd *pwent;

        pwent = getpwuid(geteuid());
        if ( pwent ) {
            /* Small memory leak, don't worry about it */
            home = strdup(pwent->pw_dir);
        }
    }

    /* Uh oh, this system is probably hosed, write to / if we can.
     */
    if ( ! home ) {
        home = "";
    }

    return home;
}
